// Implementation of a binary search tree.

#include <stdio.h>
#include <stdlib.h>

typedef struct node{
  int data;
  struct node *left;
  struct node *right;
}NODE;

NODE *InsertNode(NODE *root, int data);

int IsLeaf(NODE *root, int data); // αν το u είναι φύλλο τότε δώσε true, διαφορετικά δώσε false
void preOrderTraversal(NODE *root); // The preorder traversal extracts the value the first time it visits the node
void postOrderTraversal(NODE *root); // the value is extracted during the third visit
int Depth(NODE *root); // δώσε το βάθος.
NODE *Parent(NODE *root, int data); // επέστρεψε τον πατέρα του u
NODE *RightSibling(NODE *root, int data); // επέστρεψε τον κόμβο στα δεξιά του u
NODE *LeftSibling(NODE *root, int data); // επέστρεψε τον κόμβο στα αριστερά του u
void destroyTree(NODE *root); // Garbage collect entire binary tree


NODE *InsertNode(NODE *root, int val)
{
 if (root == NULL) {
  // create node and return it
  root = (NODE *) malloc(sizeof(NODE));
  if (root==NULL) {
   printf("Out of Memory!");
   exit(1);
  }

  root->data = val;
  root->left = NULL;
  root->right = NULL;
 }
 else {
  if (root->data >= val)
   root->left  = InsertNode(root->left, val);
  else
   root->right = InsertNode(root->right, val);  }

 return root;
}

NODE *Parent(NODE *root, int data) 
{
  NODE *child = root;
  NODE *parent;

  while(child != NULL)
  {
      if (data == child->data)
	if (child == root) {
		printf("The element is the root. ");
		return 0;
		}
	else return parent;
      else if (data > child->data) {
	  parent = child;
	  child = child->right;
	}
      else { 
	  parent = child;
 	  child = child->left;
	}
  }

  return 0;

} 

NODE *RightSibling(NODE *root, int data)
{
  NODE *child = root;
  NODE *parent;

  while(child != NULL)
  {
      if (data == child->data)
	return parent->right;
      else if (data > child->data) {
	  parent = child;
	  child = child->right;
	}
      else { 
	  parent = child;
 	  child = child->left;
	}
  }
  return 0;
}

NODE *LeftSibling(NODE *root, int data)
{
  NODE *child = root;
  NODE *parent;

  while(child != NULL)
  {
      if (data == child->data)
	return parent->left;
      else if (data > child->data) {
	  parent = child;
	  child = child->right;
	}
      else { 
	  parent = child;
 	  child = child->left;
	}
  }
  return 0;
}

int IsLeaf(NODE *root, int data)
{
  NODE *temp = root;

  while(temp != NULL)
  {
      if (data == temp->data)
	if ((temp->left == NULL) && (temp->right == NULL))
		return 1;
	else 
		return 0;
      else if (data > temp->data) 
	  temp = temp->right;
	
      else  
 	  temp = temp->left;
  }

  return 0;

}

int Depth(NODE *root)
{
  if (root==NULL) { 
    return(0); 
  } 
  else { 
    // compute the depth of each subtree 
    int lDepth = Depth(root->left); 
    int rDepth = Depth(root->right); 
    // use the larger one 
    if (lDepth > rDepth) return(lDepth+1); 
    else return(rDepth+1); 
  } 
}

void destroyTree(struct node *currNode)
{
  if (currNode != NULL)
  {
    destroyTree(currNode->left);  
    destroyTree(currNode->right); 
    free(currNode);                  
  }
}

void preOrderTraversal(NODE *root) {

 if (root == NULL)
  return;

 printf("%d, ", root->data);

 preOrderTraversal(root->left);
 preOrderTraversal(root->right);
}


void postOrderTraversal(NODE *root) {

 if (root == NULL)
  return;

 postOrderTraversal(root->left);
 postOrderTraversal(root->right);

 printf("%d, ", root->data);
}

int main()
{
 NODE *root, *temp;
 int A[] = {37,24,42,6,40,60,80};
 int n = sizeof(A) / sizeof(int);
 int i;

 root = NULL;

 for (i=0; i<n; i++) {
  printf("Inserting %d\n", A[i]);
  root = InsertNode(root, A[i]);
 }

}

